@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2014, Parrot Foundation.
+# Copyright (C) 2001-2015, Parrot Foundation.
 
 =head1 NAME
 
@@ -86,6 +86,8 @@ These items are used from current config settings:
 
 Calls the compiler and linker on F<test_$$.c>.
 
+Returns the last error code.
+
 =cut
 
 sub cc_build {
@@ -117,6 +119,7 @@ sub cc_build {
     if ($link_result) {
         return $link_result;
     }
+    return;
 }
 
 =item C<cc_run()>
@@ -126,6 +129,8 @@ sub cc_build {
 Calls the F<test> (or F<test.exe>) executable. Any output is directed to
 F<test.out>.
 
+Returns the captured stdout, and in array context with the additional exit code.
+
 =cut
 
 sub cc_run {
@@ -146,8 +151,7 @@ sub cc_run {
     }
 
     my $output = _slurp("./$test.out");
-
-    return $output;
+    return wantarray ? ($output, $run_error) : $output;
 }
 
 =item C<cc_run_capture()>
@@ -157,6 +161,9 @@ sub cc_run {
 Same as C<cc_run()> except that warnings and errors are also directed to
 F<test.out>.
 
+Returns the captured stdout combined with stderr, and in array context
+with the additional exit code.
+
 =cut
 
 sub cc_run_capture {
@@ -165,18 +172,18 @@ sub cc_run_capture {
     my $slash   = $conf->data->get('slash');
     my $verbose = $conf->options->get('verbose');
     my $test    = 'test_' . $$;
+    my $run_error;
 
     if ( defined( $_[0] ) && length( $_[0] ) ) {
         local $" = ' ';
-        _run_command( ".${slash}$test${exe} @_", "./$test.out", "./$test.out", $verbose );
+        $run_error = _run_command( ".${slash}$test${exe} @_", "./$test.out", "./$test.out", $verbose );
     }
     else {
-        _run_command( ".${slash}$test${exe}", "./$test.out", "./$test.out", $verbose );
+        $run_error = _run_command( ".${slash}$test${exe}", "./$test.out", "./$test.out", $verbose );
     }
 
     my $output = _slurp("./$test.out");
-
-    return $output;
+    return wantarray ? ($output, $run_error) : $output;
 }
 
 =item C<cc_clean()>
@@ -192,7 +199,7 @@ sub cc_clean {    ## no critic Subroutines::RequireFinalReturn
     unlink map "test_${$}$_", qw( .c .cco .ldo .out ),
         $conf->data->get(qw( o exe )),
         # MSVC
-        qw( .exe.manifest .ilk .pdb );
+        ($^O eq 'MSWin32' ? qw( .exe.manifest .ilk .pdb ) : ());
 }
 
 =item C<shebang_mod()>
