@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2010-2011, Parrot Foundation.
+Copyright (C) 2010-2014, Parrot Foundation.
 
 =head1 NAME
 
@@ -37,7 +37,7 @@ returns a true value if this call is successful and false value otherwise.
 
 */
 
-/* TODO: This only works with the inital bytecode. After this we should use
+/* TODO: This only works with the initial bytecode. After this we should use
          Parrot_append_bytecode or something similar */
 
 PARROT_API
@@ -73,13 +73,13 @@ Parrot_api_load_bytecode_bytes(Parrot_PMC interp_pmc,
 {
     ASSERT_ARGS(Parrot_api_load_bytecode_bytes)
     EMBED_API_CALLIN(interp_pmc, interp)
-    PackFile * const pf = PackFile_new(interp, 0);
+    PackFile * const pf = Parrot_pf_new(interp, 0);
     PARROT_ASSERT(pf);
 
     Parrot_block_GC_mark(interp);
-    if (!PackFile_unpack(interp, pf, (const opcode_t *)pbc, bytecode_size)) {
+    if (!Parrot_pf_unpack(interp, pf, (const opcode_t *)pbc, bytecode_size)) {
         Parrot_unblock_GC_mark(interp);
-        Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_MALFORMED_PACKFILE,
+        Parrot_ex_throw_from_c_noargs(interp, EXCEPTION_MALFORMED_PACKFILE,
             "Could not unpack packfile");
     }
     *pbcpmc = Parrot_pf_get_packfile_pmc(interp, pf, STRINGNULL);
@@ -88,7 +88,7 @@ Parrot_api_load_bytecode_bytes(Parrot_PMC interp_pmc,
 }
 
 /* Load the bytecode into the interpreter, but don't execute it */
-/* TODO: This only works with the inital bytecode. After this we should use
+/* TODO: This only works with the initial bytecode. After this we should use
        Parrot_append_bytecode or something similar */
 
 /*
@@ -120,7 +120,7 @@ Parrot_api_ready_bytecode(Parrot_PMC interp_pmc, Parrot_PMC pbc,
     }
 
     if (!pf)
-        Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_UNEXPECTED_NULL,
+        Parrot_ex_throw_from_c_noargs(interp, EXCEPTION_UNEXPECTED_NULL,
             "Could not get packfile.");
     if (pf->cur_cs)
         Parrot_pf_set_current_packfile(interp, pbc);
@@ -183,7 +183,7 @@ Parrot_api_disassemble_bytecode(Parrot_PMC interp_pmc, Parrot_PMC pbc,
     EMBED_API_CALLIN(interp_pmc, interp)
     PackFile * const pf = (PackFile *)VTABLE_get_pointer(interp, pbc);
     if (!pf)
-        Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_UNEXPECTED_NULL,
+        Parrot_ex_throw_from_c_noargs(interp, EXCEPTION_UNEXPECTED_NULL,
             "Could not get packfile.");
     if (pf->cur_cs)
         Parrot_pf_set_current_packfile(interp, pbc);
@@ -212,7 +212,7 @@ Parrot_api_serialize_bytecode_pmc(Parrot_PMC interp_pmc, Parrot_PMC pbc,
     EMBED_API_CALLIN(interp_pmc, interp)
     PackFile * const pf = (PackFile *)VTABLE_get_pointer(interp, pbc);
     if (!pf)
-        Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_UNEXPECTED_NULL,
+        Parrot_ex_throw_from_c_noargs(interp, EXCEPTION_UNEXPECTED_NULL,
             "Could not get packfile.");
     *bc = Parrot_pf_serialize(interp, pf);
     EMBED_API_CALLOUT(interp_pmc, interp)
